/*
 *  Copyright(c)2004 Forward Computing and Control Pty. Ltd.
 *  ACN 003 669 994, NSW, Australia      All rights Reserved
 *
 *   Written by Dr. M.P. Ford
 *
  <H1>Licence Agreement</H1>
<PRE>FutureTalker package Copyright (c) 2004 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source and/or compiled .class of this package, with or without
modification, is permitted subject to the limitation of libability below.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.</PRE>
 *
 */
package au.com.forward.futureTalker.test;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.CancellationException;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Executors;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import au.com.forward.futureTalker.FutureTalker;
import au.com.forward.futureTalker.IFutureListener;




/**
 *  finds a prime using Java's BigInteger.isProbablePrime()
 *
 * @author     matthew ford
 * @created    22 December 2004
 */
class Task implements Callable<Integer> {
  static Random random = new Random(1);
  String name;
  
  /**
   *
   */
  Task(String name) {
    this.name = name;
  }
  
  String getName() {
    return name==null?"":name;
  }
  
  
  public Integer call() throws Exception {
    
    // do some work here
    // check for interrupts every now and then
    FutureTalker.ifInterruptedStop();
    
    int rnd = random.nextInt(3000);
    if (rnd<1000) {
      throw new Exception("Delay too small, "+rnd);
    }
    
    FutureTalker.ifInterruptedStop();
    Thread.currentThread().sleep(rnd);
    return new Integer(rnd);
  }
  
}



/**
 *
 * @author     matthew ford
 * @created    October 21, 2001
 */
public class TalkerTest implements IFutureListener<Integer> {
  
  static CountDownLatch latch;
  
  ThreadPoolExecutor threadPool;
  
  /**
   *  The main program
   *
   * @param  args  The command line arguments.
   *               Could be used to pass in the number and the certainty,
   *               but not implemented here.
   */
  public static void main(String[] args) {
    latch = new CountDownLatch(10);
    new TalkerTest().start();
  }
  
  /**
   */
  public void start() {
    
    threadPool = (ThreadPoolExecutor)Executors.newFixedThreadPool(3);
    startTask(); // start a few tasks
    startTask();
    startTask();
    startTask();
    FutureTalker futureTalker = startTask();
    futureTalker.cancel(true);  // cancel one of them
    try {
      latch.await(); // wait until all the tasks have been started
    } catch (InterruptedException iex) {
      // interrupted
    }
    
    System.out.println("All tasks created. Shutting Down.");
    // shutdown the threadPool and reject or cancel any un-run tasks
    FutureTalker.shutdownNow(threadPool); // ignore returned list
    
  }
  
  protected FutureTalker startTask(){
    FutureTalker<Integer> futureTalker = null;
    synchronized(latch) {
      long count = latch.getCount();
      if (count <= 0) {
        return null;
      }
      futureTalker = new FutureTalker<Integer>(new Task(""+count));
      futureTalker.addListener(this);
      // start another one
      try {
        threadPool.execute(futureTalker);
      } catch (RejectedExecutionException rex) {
        futureTalker.rejected(rex);  // failed to execute set task to error and notifies listeners
      }
      latch.countDown();
      return futureTalker;
    }
  }
  
  
  // Listener methods
  /**
   * Called on normal completion
   * A call to this method implies the task's run()
   * method has exited.
   */
  public void futureResult(Integer result, FutureTalker<Integer> talker) {
    Task task = (Task)talker.getCallable();
    System.out.println("Task "+task.getName() + " returned "+ result );
    startTask();
  }
  
  /**
   * Called if cancelled
   * Note: You need to provide a rejecthandler and
   * override ShutDownNow if you want this to be called
   * on rejection and shutdown.
   * A call to this method does NOT imply the task's call()
   * method ever been called.
   */
  public void futureCancelled(CancellationException t, FutureTalker<Integer> talker) {
    Task task = (Task)talker.getCallable();
    System.out.println("Task "+task.getName()+ " was cancelled. "
            +t.getClass().getName()+"  Message:"+t.getMessage());
    startTask();
  }
  
  /**
   * Called on error and passed the throwable that
   * was wrapped in ExecutionException
   * A call to this method implies the task's run()
   * method has exited.
   */
  public void futureError(Throwable t, FutureTalker<Integer> talker) {
    Task task = (Task)talker.getCallable();
    String msg = t.getMessage();
    System.out.println("Task "+task.getName()+ " stopped due to error. "
            +t.getClass().getName()+"  Message:"+t.getMessage());
    //start another one
    startTask();
  }
  
}